From d2fac8ced47e883c7b809a4dc3612d35987f460c Mon Sep 17 00:00:00 2001
From: "Andrey V. Lepikhov" <a.lepikhov@postgrespro.ru>
Date: Fri, 27 Oct 2023 17:55:04 +0700
Subject: [PATCH] Don't alter parse tree during the procedure of outer join
 reduction.

---
 src/backend/optimizer/prep/prepjointree.c | 16 ++++++++--------
 src/backend/rewrite/rewriteManip.c        | 10 +++++++++-
 src/include/rewrite/rewriteManip.h        |  5 ++++-
 3 files changed, 21 insertions(+), 10 deletions(-)

diff --git a/src/backend/optimizer/prep/prepjointree.c b/src/backend/optimizer/prep/prepjointree.c
index 73ff40721c..06a7636403 100644
--- a/src/backend/optimizer/prep/prepjointree.c
+++ b/src/backend/optimizer/prep/prepjointree.c
@@ -2687,14 +2687,14 @@ reduce_outer_joins(PlannerInfo *root)
 	if (!bms_is_empty(state2.inner_reduced))
 	{
 		root->parse = (Query *)
-			remove_nulling_relids((Node *) root->parse,
+			remove_nulling_relids_ext((Node *) root->parse,
 								  state2.inner_reduced,
-								  NULL);
+								  NULL, QTW_DONT_COPY_QUERY);
 		/* There could be references in the append_rel_list, too */
 		root->append_rel_list = (List *)
-			remove_nulling_relids((Node *) root->append_rel_list,
+			remove_nulling_relids_ext((Node *) root->append_rel_list,
 								  state2.inner_reduced,
-								  NULL);
+								  NULL, QTW_DONT_COPY_QUERY);
 	}
 
 	/*
@@ -2707,13 +2707,13 @@ reduce_outer_joins(PlannerInfo *root)
 		Relids		full_join_relids = bms_make_singleton(statep->full_join_rti);
 
 		root->parse = (Query *)
-			remove_nulling_relids((Node *) root->parse,
+			remove_nulling_relids_ext((Node *) root->parse,
 								  full_join_relids,
-								  statep->unreduced_side);
+								  statep->unreduced_side, QTW_DONT_COPY_QUERY);
 		root->append_rel_list = (List *)
-			remove_nulling_relids((Node *) root->append_rel_list,
+			remove_nulling_relids_ext((Node *) root->append_rel_list,
 								  full_join_relids,
-								  statep->unreduced_side);
+								  statep->unreduced_side, QTW_DONT_COPY_QUERY);
 	}
 }
 
diff --git a/src/backend/rewrite/rewriteManip.c b/src/backend/rewrite/rewriteManip.c
index 32bd2f1dc9..1fdfb05cfb 100644
--- a/src/backend/rewrite/rewriteManip.c
+++ b/src/backend/rewrite/rewriteManip.c
@@ -1226,6 +1226,14 @@ Node *
 remove_nulling_relids(Node *node,
 					  const Bitmapset *removable_relids,
 					  const Bitmapset *except_relids)
+{
+	return remove_nulling_relids_ext(node, removable_relids, except_relids, 0);
+
+}
+Node *
+remove_nulling_relids_ext(Node *node,
+						  const Bitmapset *removable_relids,
+						  const Bitmapset *except_relids, int flags)
 {
 	remove_nulling_relids_context context;
 
@@ -1235,7 +1243,7 @@ remove_nulling_relids(Node *node,
 	return query_or_expression_tree_mutator(node,
 											remove_nulling_relids_mutator,
 											&context,
-											0);
+											flags);
 }
 
 static Node *
diff --git a/src/include/rewrite/rewriteManip.h b/src/include/rewrite/rewriteManip.h
index ca12780bc7..6bc7e6f36e 100644
--- a/src/include/rewrite/rewriteManip.h
+++ b/src/include/rewrite/rewriteManip.h
@@ -71,7 +71,10 @@ extern Node *add_nulling_relids(Node *node,
 extern Node *remove_nulling_relids(Node *node,
 								   const Bitmapset *removable_relids,
 								   const Bitmapset *except_relids);
-
+extern Node *remove_nulling_relids_ext(Node *node,
+									   const Bitmapset *removable_relids,
+									   const Bitmapset *except_relids,
+									   int flags);
 extern Node *replace_rte_variables(Node *node,
 								   int target_varno, int sublevels_up,
 								   replace_rte_variables_callback callback,
-- 
2.42.0

